/*
 * OLED C0245QGLC low-level initialization
 *
 * (c) 2009 Artec Design LLC, Tallinn, Estonia
 * Anti Sullin <anti.sullin@artecdesign.ee>
 *
 * SPI part is based on:
 * (c) Copyright 2005
 * M. Amine SAYA, ATMEL Rousset, France.
 * Driver for AT91SAM9261 SPI
 * Based on at91rm9200 spi driver from H. Ikdoumi, ATMEL Rousset.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

#include <config.h>
#include <common.h>

#ifdef CONFIG_LCD_C0283QGLC

#include <asm/arch/hardware.h>
#include <asm/arch/pio.h>

#define AT91C_SPI_LCD_CLK	2500000		/* SPI Clock*/

#define AT91C_SPI_LCD_CS	2
#define AT91C_SPI_PCS_LCD	(0xF & (~(1<<AT91C_SPI_LCD_CS)))


void AT91F_LCD_SpiInit(void)
{
	volatile unsigned int dummy;

	/* Configure PIOs for SPI1 */
	AT91F_PIO_CfgPeriph(AT91C_BASE_PIOA, 0, 1<<25);
	AT91F_PIO_CfgPeriph(AT91C_BASE_PIOB,
			(AT91C_PB30_MISO1 | AT91C_PB31_MOSI1 | AT91C_PB29_SPCK1 ),
			0);


	/* Enable Clock */
	AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_SPI1;

	/* Reset the SPI */
	AT91C_BASE_SPI1->SPI_CR = AT91C_SPI_SWRST;

	/* Configure SPI in Master Mode with No CS selected */
	AT91C_BASE_SPI1->SPI_MR = AT91C_SPI_MSTR | AT91C_SPI_MODFDIS | AT91C_SPI_PCS;

	AT91C_BASE_SPI1->SPI_CSR[AT91C_SPI_LCD_CS] = (AT91C_SPI_NCPHA | (AT91C_MASTER_CLOCK / AT91C_SPI_LCD_CLK) << 8);

	/* SPI_Enable */
	AT91C_BASE_SPI1->SPI_CR = AT91C_SPI_SPIEN;

	while(!(AT91C_BASE_SPI1->SPI_SR & AT91C_SPI_SPIENS));

	udelay(500000);
	dummy = AT91C_BASE_SPI1->SPI_SR;

	AT91C_BASE_SPI1->SPI_MR = (AT91C_BASE_SPI1->SPI_MR & ~AT91C_SPI_PCS) | ((AT91C_SPI_PCS_LCD<<16) & AT91C_SPI_PCS);
}

void AT91F_LCD_SpiSend(char cdata, char end)
{
	unsigned int timeout;
	unsigned int data = cdata;

	if (end) data |= 1<<24;

	AT91C_BASE_SPI1->SPI_TDR = data;

	timeout = get_timer(0);

	while(!(AT91C_BASE_SPI1->SPI_SR & AT91C_SPI_TDRE) && (get_timer(timeout) < CFG_SPI_WRITE_TOUT));

	if (get_timer(timeout) >= CFG_SPI_WRITE_TOUT) {
		printf("SPI timeout!\n");
	}
}

void Set_LCD_REG( unsigned short addr, unsigned short cmd )
{
	// Set index register
	AT91F_LCD_SpiSend(0x70, 0);
	AT91F_LCD_SpiSend(0x00, 0);
	AT91F_LCD_SpiSend(addr, 1);

	udelay(10);

	// Set data register
	AT91F_LCD_SpiSend(0x70 | 0x02, 0);
	AT91F_LCD_SpiSend((cmd >> 8) & 0xFF, 0);
	AT91F_LCD_SpiSend(cmd & 0xFF, 1);

	udelay(50);
}

void AT91F_LCD_SpiStop(void)
{
	/* Configure PIOs for SPI1 */
	AT91F_PIO_SetOutput(AT91C_BASE_PIOA, 1<<25);
	AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, 1<<25);

	/* Reset the SPI */
	AT91C_BASE_SPI1->SPI_CR = AT91C_SPI_SWRST;

	/* Disable Clock */
	AT91C_BASE_PMC->PMC_PCDR = 1 << AT91C_ID_SPI1;
}

void oled_C0283QGLC_init(void)
{
	AT91F_LCD_SpiInit();

	// no rotation
	Set_LCD_REG(0x03, 0x8100);
	// RGB interface mode, polarities
	Set_LCD_REG(0x02, 0x0192);

	// standby off
	Set_LCD_REG(0x10, 0x0000);
	// DCDC configuration
	Set_LCD_REG(0xF8, 0x000D);
	Set_LCD_REG(0xF9, 0x000F);
	// unblank
	Set_LCD_REG(0x05, 0x0001);

	// Gamma
	Set_LCD_REG(0x70, 0x378E);
	Set_LCD_REG(0x71, 0x378E);
	Set_LCD_REG(0x72, 0x3C8E);
	Set_LCD_REG(0x73, 0x1A14);
	Set_LCD_REG(0x74, 0x1A0B);
	Set_LCD_REG(0x75, 0x1F15);
	Set_LCD_REG(0x76, 0x1611);
	Set_LCD_REG(0x77, 0x211B);
	Set_LCD_REG(0x78, 0x2110);

	// hw bug? somehow these registers shift the RGB input.
	// The RGB input is horizontally (vertically on device)
	// shifted 1 pixel to left - meaning the first pixel from
	// CPU goes to second position on screen. Somehow this
	// fixes it.
	Set_LCD_REG(0x20, 0x00EF);
	Set_LCD_REG(0x21, 0x013F);

	// Set index register back to 0x22
	AT91F_LCD_SpiSend(0x70, 0);
	AT91F_LCD_SpiSend(0x00, 0);
	AT91F_LCD_SpiSend(0x22, 1);

	udelay(25);

	AT91F_LCD_SpiStop();
}

#endif
