/*
 * Artec Design board SPATHA
 * Anti Sullin, Artec Design LLC
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/arch/hardware.h>
#include <asm/arch/pio.h>

#include <command.h>

#include <net.h>
#include <beep.h>

#define AT91C_SDRAM 		((volatile unsigned int *)0x20000000)
#define AT91C_RTT_PRESCAL       33

int set_led(char led);

/* ------------------------------------------------------------------------- */
/*
 * Miscelaneous platform dependent initialisations
 */
int board_init(void) {
	DECLARE_GLOBAL_DATA_PTR;

	/* Enable Ctrlc */
	console_init_f();

	/* memory and cpu-speed are setup before relocation */
	/* so we do _nothing_ here */

#ifdef CONFIG_LCD
	gd->fb_base = (unsigned long) AT91C_IRAM;
#endif

	/* arch number of AT91SAM9261EK-Board */
	gd->bd->bi_arch_number = MACH_TYPE_AT91SAM9261EK;
	/* adress of boot parameters */
	gd->bd->bi_boot_params = PHYS_SDRAM + 0x100;

	// Enable the PIOs
	AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_PIOA;
	AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_PIOB;
	AT91C_BASE_PMC->PMC_PCER = 1 << AT91C_ID_PIOC;

	// reset the RSTC_RMR. Embedded RomBOOT writes there garbage
	AT91C_BASE_RSTC->RSTC_RMR = 0xA5000000;

	// Disable the usb pullup. this disconnects sam-ba and we can connect later other gadget driver
	AT91C_BASE_MATRIX->MATRIX_USBPCR = 0;

	// Turn on the orange led to indicate the user that something is happening
	set_led('o');

	// If not in USB mode, turn modem on
	if (!(AT91F_PIO_GetInput(AT91C_BASE_PIOB) & (1<<21))) {
		AT91F_PIO_CfgOutput(AT91C_BASE_PIOC, (1<<18)); // GPRS_PWR
		AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, (1<<17)|(1<<18)|(1<<11)); // GPRS_ON, RST, DTR
		AT91F_PIO_SetOutput(AT91C_BASE_PIOC, (1<<18)); // GPRS_PWR hi
		AT91F_PIO_SetOutput(AT91C_BASE_PIOA, (1<<17)); // GPRS_ON hi
	}

	return 0;
}

int AT91F_Serial_Hardware_Init(void) {
#ifdef CONFIG_DBGU
	*AT91C_PIOA_PDR = AT91C_PA10_DTXD | AT91C_PA9_DRXD; /* PA 10 & 9 */
	*AT91C_PMC_PCER = 1 << AT91C_ID_SYS; /* enable clock */
#endif
}

int dram_init(void) {
	DECLARE_GLOBAL_DATA_PTR;

	gd->bd->bi_dram[0].start = PHYS_SDRAM;
	gd->bd->bi_dram[0].size = PHYS_SDRAM_SIZE;
	return 0;
}

int board_late_init(void) {
	DECLARE_GLOBAL_DATA_PTR;
	beeper_init();

#ifdef CONFIG_DISABLE_WATCHDOG
	// Disable the watchdog
	AT91C_BASE_SYS->WDTC_WDMR = 1<<15;
#endif

	return 0;
}

void oled_power_on(void) {
	/* Configure PA12 in pio to enable OLED */
	*AT91C_PIOA_PER = AT91C_PIO_PA28;
	*AT91C_PIOA_OER = AT91C_PIO_PA28;
	*AT91C_PIOA_SODR = AT91C_PIO_PA28;
}

void oled_power_off(void) {
	*AT91C_PIOA_CODR = AT91C_PIO_PA28;
}

int do_oledpwr(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]) {
	if (argc != 2)
		return -1;

	switch (argv[1][0]) {
	case '1':
		oled_power_on();
		return 0;
	case '0':
		oled_power_off();
		return 0;
	}
	return -1;
}

U_BOOT_CMD(oledpwr, 2, 1, do_oledpwr,
		"oledpwr - switch OLED display power supply on/off\n",
		"");
int set_led(char led) {
	switch (led) {
		case '0':
			AT91F_PIO_CfgInput(AT91C_BASE_PIOA, AT91C_PIO_PA14);
			AT91F_PIO_CfgInput(AT91C_BASE_PIOA, AT91C_PIO_PA24);
			return 0;
		case 'R':
		case 'r':
			AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, AT91C_PIO_PA14);
			AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, AT91C_PIO_PA14);
			AT91F_PIO_CfgInput(AT91C_BASE_PIOA, AT91C_PIO_PA24);
			return 0;
		case 'G':
		case 'g':
			AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, AT91C_PIO_PA24);
			AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, AT91C_PIO_PA24);
			AT91F_PIO_CfgInput(AT91C_BASE_PIOA, AT91C_PIO_PA14);
			return 0;
		case 'O':
		case 'o':
			AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, AT91C_PIO_PA14);
			AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, AT91C_PIO_PA14);
			AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, AT91C_PIO_PA24);
			AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, AT91C_PIO_PA24);
			return 0;
		}
	return -1;
}

int do_led(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]) {
	if (argc != 2)
		return -1;
	return set_led(argv[1][0]);
}

U_BOOT_CMD(led, 2, 1, do_led,
		"led - configure leds\n",
		"configure leds to {0,r,g,o}");

int do_wdtdis(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]) {
	AT91C_BASE_SYS->WDTC_WDMR = 1 << 15;
	return 0;
}

U_BOOT_CMD(wdtdis, 1, 1, do_wdtdis,
		"wdtdis - disable watchdog\n",
		"");
