/*
 * (C) Copyright 2005
 * M. Amine SAYA ATMEL Rousset, France.
 *
 * (C) Copyright 2003
 * Texas Instruments <www.ti.com>
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Alex Zuepke <azu@sysgo.de>
 *
 * (C) Copyright 2002-2004
 * Gary Jennejohn, DENX Software Engineering, <gj@denx.de>
 *
 * (C) Copyright 2004
 * Philippe Robin, ARM Ltd. <philippe.robin@arm.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/arch/hardware.h>

#define TIMER_LOAD_VAL 0xFFFFF

/* macro to read the 20-bit wide AT91SAM9263 PITC value*/
#define READ_RESET_TIMER (p_pitc->PITC_PIVR)
#define READ_TIMER (p_pitc->PITC_PIIR)

AT91PS_PITC	p_pitc;

/* nothing really to do with interrupts, just starts up a counter. */
int interrupt_init (void)
{
	unsigned int tmp;

	p_pitc = AT91C_BASE_PITC;

	/* Enable PITC Clock
	The clock is already enabled for system controller in boot */
	*AT91C_PMC_PCER = 1 << AT91C_ID_SYS;

	/* Enable PITC */
	p_pitc->PITC_PIMR = AT91C_PITC_PITEN | TIMER_LOAD_VAL;

	tmp = READ_RESET_TIMER;

	return 0;
}


/*
 * timer without interrupts
 */
ulong get_timer (ulong base)
{
	return READ_TIMER - base;
}


/* delay x useconds AND perserve advance timstamp value */
void udelay (unsigned long usec)
{
	unsigned long timeout;
	unsigned long start;

	do { // up to 600000us @ 100MHz at once
		if (usec <= 600000) {
			timeout = usec * (AT91C_MASTER_CLOCK/16/1000);
			usec = 0;
		} else {
			timeout = 600000 * (AT91C_MASTER_CLOCK/16/1000);
			usec -= 600000;
		}
		timeout /= 1000;
		if (timeout != 0xFFFFFFFF) timeout += 1;

		start = READ_TIMER;

		while(READ_TIMER - start < timeout);
	} while (usec);
}

/*
 * This function is derived from PowerPC code (read timebase as long long).
 * On ARM it just returns the timer value.
 */
unsigned long long get_ticks(void)
{
	return READ_TIMER;
}

/*
 * This function is derived from PowerPC code (timebase clock frequency).
 * On ARM it returns the number of timer ticks per second.
 */
ulong get_tbclk (void)
{
	return CFG_HZ;
}

/*
 * Reset the cpu by setting up the Reset Controller
 * on the SAM9263EK board
 */
void reset_cpu (ulong ignored)
{
	AT91PS_RSTC	p_rstc = AT91C_BASE_RSTC;

	/* Use Reset Controller */
	p_rstc->RSTC_RCR = (AT91C_RSTC_KEY | AT91C_RSTC_PROCRST | AT91C_RSTC_PERRST);
	
	while (1);
	/* Never reached */
}

