/*
 * (C) Copyright 2005
 * M. Amine SAYA ATMEL Rousset, France.
 *
 * Rick Bronson <rick@efn.org>
 *
 * Configuation settings for the AT91SAM9261EK board.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H

/* ARM asynchronous clock */
#define AT91C_MAIN_CLOCK	200010000	/* from 13.560 MHz crystal (13560000 / 4 * 59) */
#define AT91C_MASTER_CLOCK	100005000	/* peripheral clock (AT91C_MASTER_CLOCK / 2) */

#define AT91_SLOW_CLOCK		32768	/* slow clock */

/* PIT timer frequency */
#define CFG_HZ			(AT91C_MASTER_CLOCK / 16)

#define CONFIG_SPATHA			1	/* on an Artec SPATHA Board */
#define CONFIG_AT91SAM9261		1	/* AT91SAM9261 CPU */
#undef  CONFIG_USE_IRQ			        /* we don't need IRQ/FIQ stuff */
#define CONFIG_CMDLINE_TAG	        1	/* enable passing of ATAGs	*/
#define CONFIG_SETUP_MEMORY_TAGS        1
#define CONFIG_INITRD_TAG	        1

/* define this to include the functionality of boot.bin in u-boot */
#undef CONFIG_SKIP_LOWLEVEL_INIT
#undef CONFIG_SKIP_RELOCATE_UBOOT
/*
 * Size of malloc() pool
 */
#define CFG_MALLOC_LEN	(CFG_ENV_SIZE + 128*1024)
#define CFG_GBL_DATA_SIZE	128	/* size in bytes reserved for initial data */

#define CONFIG_BAUDRATE         115200

/*
 * Hardware drivers
 */

/* define one of these to choose the DBGU, USART0  or USART1 as console */
#define CONFIG_DBGU       1
#undef CONFIG_USART0
#undef CONFIG_USART1
#undef CONFIG_USART2

#undef	CONFIG_HWFLOW			/* don't include RTS/CTS flow control support	*/

#undef	CONFIG_MODEM_SUPPORT		/* disable modem initialization stuff */

#define CONFIG_BOOTDELAY      1
#define CONFIG_ZERO_BOOTDELAY_CHECK 1

#define CONFIG_BOOTSCRIPT	1
#define CONFIG_MAX_BOOTSCRIPT_CMD_COUNT 25
#define CONFIG_BOOTSCRIPT_ADDRESS 0x200FFC00

/* #define CONFIG_ENV_OVERWRITE  1 */
#define BOARD_LATE_INIT		1

#define CONFIG_COMMANDS		\
		       (CFG_CMD_FLASH | \
		        CFG_CMD_MEMORY | \
			CFG_CMD_RUN | \
                        CFG_CMD_ENV | \
			CFG_CMD_AUTOSCRIPT)

/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>


#define CONFIG_NR_DRAM_BANKS            1
#define PHYS_SDRAM                      0x20000000

//#define PHYS_SDRAM_SIZE                 0x4000000  /* 64 megs */
#define PHYS_SDRAM_SIZE                 0x2000000  /* 32 megs */

// !!!! One more copy of CFG_LOAD_ADDR is in config.mk !!!!
//#define CFG_LOAD_ADDR		0x23f00000  /* default load address - 64MB */
#define CFG_LOAD_ADDR		0x21f80000  /* default load address - 32MB */

#define CFG_MEMTEST_LFSR
// MEMTEST: ubootscript end to uboot beginning
// (we don't test the first MB and the last MB)
#define CFG_MEMTEST_START		0x20100000
#define CFG_MEMTEST_END			(CFG_LOAD_ADDR - CFG_MALLOC_LEN - CFG_GBL_DATA_SIZE - CONFIG_STACKSIZE)
//#define CFG_MEMTEST_END			CFG_MEMTEST_START + PHYS_SDRAM_SIZE - 262144


/*-----------------------------------------------------------------------
 * FLASH organization
 */

#define CFG_ENV_IS_IN_FLASH	1

#define CFG_FLASH_BASE		0x10000000
#define CFG_FLASH_BANKS_LIST			{0x10000000}

#define CFG_FLASH_CFI_DRIVER

#define CFG_FLASH_CFI
#define CFG_FLASH_USE_BUFFER_WRITE
#define CFG_FLASH_PROTECTION

#define CFG_MAX_FLASH_BANKS	1	/* max num of flash banks	*/
#define CFG_MAX_FLASH_SECT	300	/* max num of sects on one chip */


/* BEG ENVIRONNEMENT FLASH */
#ifdef CFG_ENV_IS_IN_FLASH
#define CFG_ENV_OFFSET		0x00020000 /* Offset of Environment Sector	*/
#define CFG_ENV_SECT_SIZE	0x20000 /* see README - env sector total size	*/
#define CFG_ENV_SIZE	0x8000 /* env total size 32KB */
#endif
/* END ENVIRONNEMENT FLASH */

/* BEG NO ENVIRONNEMENT */
#ifdef CFG_ENV_IS_NOWHERE
#define CFG_ENV_SIZE	0x4000 /* env total size	*/
#endif
/* END NO ENVIRONNEMENT */

#define CONFIG_AT91_BEEPER

/* Add LCD stuff */
#define CONFIG_LCD
#define LCD_BPP LCD_COLOR16
#define CONFIG_LCD_C0283QGLC
#define CFG_SPI_WRITE_TOUT		(50*CFG_HZ)

// bootsplash is copied to sram early, in lowlevel_init.S
#define COPY_BOOTSPLASH_TO_SRAM
#define BOOTSPLASH_ADDR 0x10040000
#define BOOTSPLASH_END_ADDR 0x10065800

/* Add USB stuff */
/*#define CONFIG_USB_OHCI			1
#define CONFIG_USB_STORAGE		1
#define CONFIG_DOS_PARTITION	        1*/
#define LITTLEENDIAN 			1

#define CFG_BAUDRATE_TABLE	{115200 , 19200, 38400, 57600, 9600 }
#define CFG_CONSOLE_IS_SERIAL
#undef  CFG_CONSOLE_IS_LCD
#define CFG_PROMPT		"> "	/* Monitor Command Prompt */
#define CFG_CBSIZE		512		/* Console I/O Buffer Size */
#define CFG_MAXARGS		32		/* max number of command args */
#define CFG_PBSIZE		(CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */


#ifndef __ASSEMBLY__
/*-----------------------------------------------------------------------
 * Board specific extension for bd_info
 *
 * This structure is embedded in the global bd_info (bd_t) structure
 * and can be used by the board specific code (eg board/...)
 */

struct bd_info_ext {
	/* helper variable for board environment handling
	 *
	 * env_crc_valid == 0    =>   uninitialised
	 * env_crc_valid  > 0    =>   environment crc in flash is valid
	 * env_crc_valid  < 0    =>   environment crc in flash is invalid
	 */
	int env_crc_valid;
};
#endif


#define CONFIG_STACKSIZE	(32*1024)	/* regular stack */

#ifdef CONFIG_USE_IRQ
#error CONFIG_USE_IRQ not supported
#endif

#endif
